#include <stdio.h>
#include <stdlib.h>
#include <handy.h>
#include "sorting.h"

#define ITEMS  100
#define SCALE  5.0  // should be greater than 1.0

/*extern*/
doubleLayer wlayer;
int count = 0;
static int slow = 0;

static void initArray(int a[], int n, int natural)
{
    int i;
    if (natural) {
        for (i = 0; i < n; i++)
            a[i] = random() % (n * (int)SCALE) + 1;
        return;
    }
    for (i = 0; i < n; i++)
        a[i] = (i + 1) * SCALE;
    for (i = 0; i < n; i++) {
        int x = random() % n;
        if (i != x) {
            int t = a[i]; a[i] = a[x]; a[x] = t;
        }
    }
}

static void drawAll(int lid, int a[], int n)
{
    int i;
    HgLClear(lid);
    HgWSetFillColor(lid, HG_DGRAY);
    for (i = 0; i < n; i++) {
        HgWBoxFill(lid, i*SCALE, 0.0, SCALE-1.0, a[i], 0);
    }
}

void replaced(int a[], int i)
{
    int lid, brk;
    lid = wlayer.hidden;
    for (brk = 0; ; brk = 1) {
        HgWSetFillColor(lid, HG_WHITE);
        HgWBoxFill(lid, i*SCALE, 0.0, SCALE, ITEMS*SCALE, 0);
        HgWSetFillColor(lid, HG_DGRAY);
        HgWBoxFill(lid, i*SCALE, 0.0, SCALE-1.0, a[i], 0);
        if (brk) break;
        lid = HgLSwitch(&wlayer);
    }
    if (slow)
        HgSleep((slow == 1) ? 0.01 : 0.07);
}

void swap(int a[], int i, int k)
{
    int lid, brk;
    int t = a[i];
    a[i] = a[k], a[k] = t;
    lid = wlayer.hidden;
    for (brk = 0; ; brk = 1) {
        HgWSetFillColor(lid, HG_WHITE);
        HgWBoxFill(lid, i*SCALE, 0.0, SCALE, ITEMS*SCALE, 0);
        HgWBoxFill(lid, k*SCALE, 0.0, SCALE, ITEMS*SCALE, 0);
        HgWSetFillColor(lid, HG_DGRAY);
        HgWBoxFill(lid, i*SCALE, 0.0, SCALE-1.0, a[i], 0);
        HgWBoxFill(lid, k*SCALE, 0.0, SCALE-1.0, a[k], 0);
        if (brk) break;
        lid = HgLSwitch(&wlayer);
    }
    if (slow)
        HgSleep(0.025 * slow);
}

static void help(const char *name) {
    fprintf(stderr, "Usage: %s [options] [seed]\n", name);
    fprintf(stderr, "Options:  -s[1-9]   Slow speed (-s9 is very slow)\n");
    fprintf(stderr, "          -w        Worst case\n");
    fprintf(stderr, "          -r        Random naturally\n");
    fprintf(stderr, "\"seed\" is for the random sequence.\n");
    fprintf(stderr, "Option -w cancels -r and seed.\n");
}

int main(int argc, char *argv[])
{
    int ac, ch, rflag = 0;
    int array[ITEMS];
    int seed = 2015;

    for (ac = 1; ac < argc; ac++) {
        if (argv[ac][0] == '-') {
            switch (argv[ac][1]) {
            case 'w': seed = 0; break; // WORST CASE
            case 's':  // slow
                slow = 1;
                if ((ch = argv[ac][2]) > '0' && ch <= '9')
                    slow = ch - '0' + 1;
                break;
            case 'r': rflag = 1; break;
            case 'h':
            default:
                 help(argv[0]); return 1;
            }
        }else {
            int n = atoi(argv[ac]);
            if (n != 0) seed = n;
        }
    }
    if (seed != 0) {
        srandom(seed);
        initArray(array, ITEMS, rflag);
    }else { // WORST CASE
        int i;
        for (i = 0; i < ITEMS; i++)
            array[i] = (ITEMS - i) * SCALE;
    }

    HgOpen(ITEMS * SCALE, ITEMS * SCALE);
    HgSetTitle(sortname);
    wlayer = HgWAddDoubleLayer(0);
    drawAll(wlayer.display, array, ITEMS);
    HgLCopy(wlayer.hidden, wlayer.display);
    count = 0;
    sort(array, ITEMS);
    printf("compare = %d\n", count);
    getchar();
    return 0;
}
